package netproj.controller;

import java.util.LinkedList;
import java.util.List;
import java.util.Scanner;
import java.util.logging.*;

import netproj.routers.Router;
import netproj.skeleton.Device;
import netproj.skeleton.Link;
import netproj.skeleton.Packet;

public class Terminal {
	public static Logger log;
	public static SimulatorSpecification ss;
	public static ConsoleHandler ch;
	
	public static void main(String args[]) throws Exception {
		Logger log = Logger.getLogger("netproj");
		 // set to FINEST to see all messages, or OFF to see none at all
		log.setLevel(Level.OFF);
		ch = new ConsoleHandler();
		 // set to FINEST to see all messages, or OFF to see none at all
		ch.setLevel(Level.OFF);
		log.addHandler(ch);
		ss = SimulatorSpecification.loadXML(args[0]);

		terminal();
	}
	
	public static void terminal() throws Exception {
		String input = new String();
		Scanner sc = new Scanner(System.in);
		boolean exit = false;
		while (!exit) { try {
			System.out.println("Enter a command (or see help)");
			input = sc.nextLine();
			if (input.equals("exit") || input.equals("q")) {
				exit = true;
			} else if(input.equals("help")) {
				System.out.println("Commands include: exit, help, set-level, list, start, send, terminal, wait ");
			} else if (input.equals("start") || (input.equals("g"))) {
				ss.start();
			} else if (input.startsWith("set-level") || input.startsWith("sl ")) {
				ch.setLevel(Level.parse(input.substring(input.indexOf(' ')+1)));
			} else if (input.startsWith("add-tcp ") || input.startsWith("atcp ")) {
				input = input.substring(input.indexOf(' ')+1);
				String[] options = input.split(" ");
				int i_buf = Integer.decode(options[1]);
				int o_buf = Integer.decode(options[2]);
				int ip = Integer.decode(options[3]);
				String mode = options[0];
				if (mode.equalsIgnoreCase("fast")) {
					ss.addFastTcp(i_buf, o_buf, ip);
				} else if (mode.equalsIgnoreCase("tahoe")){
					ss.addTcpTahoe(i_buf, o_buf, ip);
				} else {
					System.out.println("Invalid TCP mode");
				}
				
			} else if (input.startsWith("stop") || input.startsWith("st ")) {
				input = input.substring(input.indexOf(' ')+1);
				String[] options = input.split(" ");
				int ip = Integer.decode(options[0]);
				Device d_from = ss.getDevice(ip);
				d_from.stop();
			} else if (input.startsWith("wait ") || input.startsWith("w ")) {
					input = input.substring(input.indexOf(' ')+1);
					String[] options = input.split(" ");
					long time = Integer.decode(options[0]);
					System.out.println("Waiting for " + time + " ms");
					Object wObj = new Object();
					try {
						synchronized (wObj) {
							wObj.wait(time);
						}
					} catch (InterruptedException e) {
						
					}
					System.out.println(time + " ms have passed.");
			} else if (input.startsWith("disconnect") || input.startsWith("dc ")) {
				input = input.substring(input.indexOf(' ')+1);
				String[] options = input.split(" ");
				int from_ip = Integer.decode(options[0]);
				int to_ip = Integer.decode(options[1]);
				Device d_from = ss.getDevice(from_ip);
				Device d_to = ss.getDevice(to_ip);
				List <Link> fromLinks = d_from.getLinks();
				for(Link l : fromLinks) {
					for(Device d : l.getDevices()) {
						if(d.getAddress() == to_ip) {
							l.stop();
							break;
						}
					}
				}
				List <Link> toLinks = d_to.getLinks();
				for(Link l : toLinks) {
					for(Device d : l.getDevices()) {
						if(d.getAddress() == from_ip) {
							l.stop();
							break;
						}
					}
				}	
			} else if (input.startsWith("connect") || input.startsWith("c ")) {
				input = input.substring(input.indexOf(' ')+1);
				String[] options = input.split(" ");
				int from_ip = Integer.decode(options[0]);
				int to_ip = Integer.decode(options[1]);
				int bps = Integer.decode(options[2]);
				int delay = Integer.decode(options[3]);
				// is there already a link between them
				boolean found_link = false;
				Device d_from = ss.getDevice(from_ip);
				Device d_to = ss.getDevice(to_ip);
				List <Link> fromLinks = d_from.getLinks();
				for(Link l : fromLinks) {
					for(Device d : l.getDevices()) {
						if(d.getAddress() == to_ip) {
							found_link = true;
							l.setBitsPerSec(bps);
							l.setDelayMs(delay);
							l.start();
							break;
						}
					}
				}	
				List <Link> toLinks = d_to.getLinks();
				for(Link l : toLinks) {
					for(Device d : l.getDevices()) {
						if(d.getAddress() == from_ip) {
							found_link = true;
							l.setBitsPerSec(bps);
							l.setDelayMs(delay);							
							l.start();
							break;
						}
					}
				}	
				if(!found_link) {
					LinkedList<Device> devices = new LinkedList<Device>();
					devices.add(d_from);
					devices.add(d_to);
					Link l = new Link(bps, delay, devices);
					int from_gate = d_from.addLink(l);
					if (d_from instanceof Router) {
						((Router)d_from).addRoutingTableEntry(to_ip, 32, from_gate);
					}
					int to_gate = d_to.addLink(l);					
					if (d_to instanceof Router) {
						((Router)d_to).addRoutingTableEntry(from_ip, 32, to_gate);
					}
					
					ss.links.add(l);
					l.start();
				}
			} else if (input.startsWith("terminal") || input.startsWith("t ")) {
				ss.getDevice(Integer.decode(input.trim().split(" ")[1])).terminal(sc);
			} else if (input.equals("list") || input.equals("l")) {
				System.out.println("Routers: ");
				for(Router r : ss.getRouters()) {
					System.out.println("\t" + Integer.toHexString(r.getAddress()));
				}
				System.out.println("Hosts: ");
				for(Device d : ss.getDevices()) {
					System.out.println("\t" + Integer.toHexString(d.getAddress()));
				}
				System.out.println("Links: ");
				for(Link l : ss.getLinks()) {
					System.out.print("\t-");
					for (Device d : l.getDevices()) {
						System.out.print(Integer.toHexString(d.getAddress()) + "-");	
					}
					System.out.println();
				}
			} else if (input.startsWith("send ") || input.startsWith("s ")) {
				// send data between hosts
				// syntax: host ip, dest ip, size
				input = input.substring(input.indexOf(' ')+1);
				String[] options = input.split(" ");
				int from_ip = Integer.decode(options[0]);
				int to_ip = Integer.decode(options[1]);
				int size = Integer.decode(options[2]);
				Device from = null;
				boolean fromFound = false;
				for(Device d : ss.getDevices()) {
					if(d.getAddress() == from_ip) {
						from = d;
						fromFound = true;
					}
				}
				if(fromFound){
					from.broadcastPacket(new Packet(from_ip,to_ip,size));
				} else {
					System.out.println("Invalid from IP");
				}
			} else {
				System.out.println("Bad command.");
			}
		} catch(Exception e) { System.out.println("Bad command error."); throw e;}
			
		}
	}
}
