package netproj.hosts;

import java.util.logging.Logger;

import netproj.skeleton.Packet;

public class UDPSender extends Host{
	private static final Logger logger = Logger.getLogger(UDPSender.class.getCanonicalName());
	private int packetSize = 0;
	
	public UDPSender(int inputBuffSize, int outputBuffSize, int address, int packetSize) {
		super(inputBuffSize, outputBuffSize, address);
		this.packetSize = packetSize;
	}
	
	@Override
	public void processPacket(Packet pkt) {
		// Check to make sure the packet is a UDPPacket and therefore has a checksum.
		if (pkt instanceof UDPPacket) {
			// Convert to UDPPacket so we can check the checksum.
			UDPPacket upkt = (UDPPacket) pkt;
			
			// Calculate the checksum as of now so we can compare it to the original.
			short check = upkt.calcCheckSum();
		
			//If the checksum matches what it's supposed to then the packet was received properly.
			if (check == upkt.getCheckSum()) {
				logger.fine("Packet " + pkt + " received at " + Integer.toHexString(getAddress()));
			// If the checksum is wrong then the packet has been corrupted.
			} else {
				logger.fine("Error: Packet" + pkt + "corrupted at" + Integer.toHexString(getAddress()));
			}
		} else {
			// This is not an error, a host may receive many packets that are not for it. Just
			// ignore them.
			//logger.fine("Error: Packet does not have a checksum");
		}
	}
	
	/**
	 * Same as in FTPHost, except that the Packets have been changed to UDPPackets so
	 * the receivers can check the checksum.
	 */
	protected final void sendData(int dest, int size, int ttl) {
		int packetCount = size / this.packetSize;
		for(int i=0; i < packetCount; i++) {
			UDPPacket packet = new UDPPacket(this.getAddress(),dest, this.packetSize, ttl);
			packet.setCheckSum(); // Set the checksum.
			sendPacket(packet, this.getDefaultGateway());
		}
		// are we done? (was it not a multiple of the packet size)
		if (size - packetCount * this.packetSize > 0) {
			UDPPacket packet = new UDPPacket(this.getAddress(),dest, size - packetCount * this.packetSize, ttl);	
			packet.setCheckSum(); // Set the checksum.
			sendPacket(packet, this.getDefaultGateway());
		}
	}
	
	@Override
	public void handleCommand(String command) {
		if (command.equals("help")) {
			super.handleCommand("help");
			System.out.println("send <address> <size> - sends <size> bits to <address>");
		} else if (command.startsWith("send")) {
			sendData(Integer.decode(command.split(" ")[1]), Integer.decode(command.split(" ")[1]), 64);
		} else {
			super.handleCommand(command);
		}
	}
}
