package netproj.hosts.tcp;


/**
 * A FastTCP sender/receiver Host.
 * 
 * Sends "files" of certain size through the network using the FastTCP congestion control algorithm.
 * 
 * @author emil
 */
public class FastTcpHost extends TcpHost {
	final static double GAMMA = 0.5;
	final static double ALPHA = 10;
	
	private long baseRtt = Long.MAX_VALUE;
	private long lastUpdate = 0;
	
	public FastTcpHost(int inputBuffSize, int outputBuffSize, int address) {
		super(inputBuffSize, outputBuffSize, address);
		windowSize = 0;
	}
	
	/**
	 * Sends a file through the network.
	 * 
	 * Starts a new session and sends {@code size} bits to {@code dest}.
	 */
	@Override
	public void sendFile(int dest, int size) {
		// Reset stats for a new session
		baseRtt = Long.MAX_VALUE;
		lastUpdate = System.currentTimeMillis();
		
		super.sendFile(dest, size);
	}
	
	public long getBaseRtt() {
		return baseRtt;
	}
	
	/**
	 * Updates baseRtt and window size
	 */
	@Override
	protected void onAck(TcpPacket packet) {
		// Update baseRtt
		if (packet.getRtt() < baseRtt) {
			baseRtt = packet.getRtt();
		}
		
		// Update window size occasionally
		if (System.currentTimeMillis() - lastUpdate >= 2000) {
			lastUpdate = System.currentTimeMillis();
			windowSize = Math.min(2*windowSize, 
					   ((1-GAMMA) * windowSize + GAMMA * ((double)baseRtt/(double)rtt * (double)windowSize + ALPHA)));
		}
				
	}
}
