package netproj.routers;

import netproj.skeleton.Packet;

/**
 * A type of packet used specifically by {@code BFRouter} devices to learn
 * the topology of the network.  In particular, each router broadcasts
 * these packets to all neighboring links, in hopes that any recipients that
 * are routers will broadcast a reply informing the original sender of the
 * existence of a router on that link.
 * 
 * This packet is only used to determine which routers are only one hop away
 * from the original sender; after this information is received, the
 * {@code BFWeightPacket} object is used to share further routing information
 * with neighbors, building up the information to learn the rest of the topology.
 * 
 * 
 * @author nathan
 *
 */
public class BFDiscoverPacket extends Packet {
	private long timestamp;
	private int elapsedTime;
	private int sendGate; // gate through which the packet was sent at first
	
	public BFDiscoverPacket(int source, int dest, int size, int ttl) {
		super(source, dest, size, ttl);
	}
	
	public BFDiscoverPacket(int source, int dest, int size, int ttl, int elapsed) {
		super(source, dest, size, ttl);
		elapsedTime = elapsed;
	}
	
	protected BFDiscoverPacket(BFDiscoverPacket p) {
		super(p);
		timestamp = p.getTimestamp();
		elapsedTime = p.getElapsedTime();
		sendGate = p.getSendGate();
	}
	
	public Packet copy() {
		return new BFDiscoverPacket(this);
	}
	
	public int getElapsedTime() {
		return elapsedTime;
	}
	
	public long getTimestamp() {
		return timestamp;
	}
	
	public int getSendGate() {
		return sendGate;
	}
	
	public void setSendGate(int gate) {
		sendGate = gate;
	}
	
	public void setElapsedTime(int e) {
		elapsedTime = e;
	}
	
	public void recordTimestamp() {
		timestamp = System.currentTimeMillis();
	}
}
